﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Navigation;
using Microsoft.Devices;
using Microsoft.Phone.Controls;

namespace Wp7nl.Behaviors
{
  /// <summary>
  /// A behavior that shows a camera view on the background of a panel
  /// </summary>
  public class CameraViewBackgroundBehavior : Behavior<Panel>
  {
    private PhotoCamera camera;
    private PhoneApplicationFrame parentPage;
    private Uri pageSource;
    private VideoBrush backgroundBrush;

    // Setup and cleanup according to http://bit.ly/dZb6D9

    #region Setup
    protected override void OnAttached()
    {
      base.OnAttached();
      AssociatedObject.Loaded += AssociatedObjectLoaded;
      AssociatedObject.Unloaded += AssociatedObjectUnloaded;
    }

    private void AssociatedObjectLoaded(object sender, RoutedEventArgs e)
    {
      DoInit();
    }
    #endregion

    #region Cleanup
    private bool isCleanedUp;

    private void Cleanup()
    {
      if (!isCleanedUp)
      {
        isCleanedUp = true;
        AssociatedObject.Loaded -= AssociatedObjectLoaded;
        AssociatedObject.Unloaded -= AssociatedObjectUnloaded;
        DoCleanup();
      }
    }

    protected override void OnDetaching()
    {
      Cleanup();
      base.OnDetaching();
    }

    private void AssociatedObjectUnloaded(object sender, RoutedEventArgs e)
    {
      Cleanup();
    }
    #endregion

    /// <summary>
    /// Initializes the behavior
    /// </summary>
    private void DoInit()
    {
      // Find the page this control is on and listen to its orientation changed events
      if( parentPage == null)
      {
        parentPage = Application.Current.RootVisual as PhoneApplicationFrame;
        pageSource = parentPage.CurrentSource;
        parentPage.Navigated += ParentPageNavigated;

      }
      parentPage.OrientationChanged += ParentPageOrientationChanged;

      camera = new PhotoCamera();

      // Create a video brush with the right parameters
      backgroundBrush = new VideoBrush();
      backgroundBrush.Stretch = Stretch.UniformToFill;
      backgroundBrush.AlignmentX = AlignmentX.Left;
      backgroundBrush.AlignmentY = AlignmentY.Top;

      // Set the video brush to the background of the panel 
      // and and do an initial display
      AssociatedObject.Background = backgroundBrush;
      backgroundBrush.SetSource(camera);
      SetVideoOrientation(parentPage.Orientation);
    }

    /// <summary>
    /// Safely detach all extra events
    /// </summary>
    private void DoCleanup()
    {
      parentPage.OrientationChanged -= ParentPageOrientationChanged;
      camera.Dispose();
    }

    private void ParentPageNavigated(object sender, NavigationEventArgs e)
    {
      if (e.NavigationMode == NavigationMode.Back && e.Uri.Equals(pageSource))
      {
        isCleanedUp = false;
        AssociatedObject.Loaded += AssociatedObjectLoaded;
        AssociatedObject.Unloaded += AssociatedObjectUnloaded;
      }
    }

    private void ParentPageOrientationChanged(object sender, OrientationChangedEventArgs e)
    {
      SetVideoOrientation(e.Orientation);
    }

    /// <summary>
    /// Sets background video brush parameters based upon page orientation
    /// </summary>
    /// <param name="orientation"></param>
    private void SetVideoOrientation(PageOrientation orientation)
    {
      System.Diagnostics.Debug.WriteLine("Switching to {0}", orientation);
      switch (orientation)
      {
        case PageOrientation.PortraitUp:
          backgroundBrush.Transform = new CompositeTransform { Rotation = 90, TranslateX = 480 };
          break;
        case PageOrientation.LandscapeLeft:
          backgroundBrush.Transform = null;
          break;
        case PageOrientation.LandscapeRight:
          if (Microsoft.Phone.Shell.SystemTray.IsVisible)
          {
            backgroundBrush.Transform = new CompositeTransform { Rotation = 180, TranslateX = 728, TranslateY = 480 };
          }
          else
          {
            backgroundBrush.Transform = new CompositeTransform { Rotation = 180, TranslateX = 800, TranslateY = 480 };
          }
          break;
      }
    }
  }
}
